package org.nakedobjects.nos.remote.xstream;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.nof.reflect.remote.IllegalRequestException;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nos.remote.command.ConnectionException;
import org.nakedobjects.nos.remote.command.Request;
import org.nakedobjects.nos.remote.command.socket.ServerConnection;

import com.thoughtworks.xstream.XStream;


class XStreamServerConnection extends ServerConnection {
    private static final Logger LOG = Logger.getLogger(XStreamServerConnection.class);
    private final ObjectOutputStream output;
    private final ObjectInputStream input;
    private final XStream xstream = new XStream();

    public XStreamServerConnection(final InputStream input, OutputStream output, final Distribution server) {
        super(server);
        try {
            this.output = new ObjectOutputStream(output);
            this.input = new ObjectInputStream(input);
        } catch (IOException e) {
            throw new ConnectionException(e);
        } 
    }

    protected Request awaitRequest() throws IOException {
        try {
            String requestData = (String) input.readObject();
            LOG.debug("request received \n" + requestData);
            Request request = (Request) xstream.fromXML(requestData);
            return request;
        } catch (ClassNotFoundException e) {
            throw new IllegalRequestException("unknown class received; closing connection: " + e.getMessage(), e);
        }
    }

    protected void sendError(NakedObjectRuntimeException exception) throws IOException {
        String responseData = xstream.toXML(exception);
        sendData(responseData);
    }

    protected void sendResponse(Object response) throws IOException {
        String responseData = xstream.toXML(response);
        sendData(responseData);
    }

    private void sendData(String responseData) throws IOException {
        LOG.debug("send response \n" + responseData);
        output.writeObject(responseData);
        output.flush();
    }

}
// Copyright (c) Naked Objects Group Ltd.
